@@ -13,23 +13,25 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#include "pytorch_extension_utils.h"
+#include <cuda_fp16.h>
+
 #include "tensorrt_llm/common/cudaUtils.h"
 #include "tensorrt_llm/kernels/quantization.h"
-#include "tensorrt_llm/thop/thUtils.h"
-// #include "tensorrt_llm/runtime/torchUtils.h"
+#include "tensorrt_llm/thop/utils.h"
+#include "tvm/ffi/container/array.h"
 
-#include <ATen/ATen.h>
-#include <ATen/Tensor.h>
-#include <ATen/cuda/EmptyTensor.h>
-#include <cuda_fp16.h>
+#if CUDA_VERSION >= 12080
 #include <cuda_fp4.h>
+#endif
 #include <cuda_fp8.h>
+#include <tvm/ffi/container/map.h>
 
+#include <cmath>
 #include <cstdint>
-// namespace th = torch;
 
-namespace torch_ext {
+using tvm::ffi::Array;
+using tvm::ffi::Map;
+using tvm::ffi::Optional;
 
 static int getExp(float v) {
   int vIntRepr;
@@ -76,7 +78,7 @@ static int const kE2M1Count = 8;
 
 uint8_t floatToE2M1(float value) {
   float absValue = fabs(value);
-  TORCH_CHECK_LT(absValue, 8.0f);
+  TVM_FFI_ICHECK_LT(absValue, 8.0f);
   uint8_t result = getSign(value) ? 8 : 0;
   int fp4AbsValue = kE2M1Count - 1;
   for (; fp4AbsValue > 0; --fp4AbsValue) {
@@ -138,16 +140,18 @@ int computeSFIndex(int rowIdx, int colIdx, int totalRow, int totalColumn,
 // Interleave (and possibly pad) the weights block scaling factor.
 // blockScale: [num_experts, rows, cols] or [rows, cols]
 // Return: num_experts * pad_up(rows, 128) * pad_up(cols, 4)
-at::Tensor BlockScaleInterleave(at::Tensor const& blockScale) {
-  bool is_cuda = blockScale.device().is_cuda();
+void BlockScaleInterleave(TensorView blockScale, TensorView interleavedBlockScale) {
+  bool is_cuda = (blockScale->device.device_type == kDLCUDA);
   if (is_cuda) {
-    CHECK_INPUT_TYPE(blockScale, SF_DTYPE);
+    CHECK_CUDA(blockScale);
   } else {
-    CHECK_CPU_INPUT(blockScale, SF_DTYPE);
+    CHECK_CPU(blockScale);
   }
-  auto blockScaleShape = blockScale.sizes();
-  TORCH_CHECK(blockScaleShape.size() == 2 || blockScaleShape.size() == 3,
-              "Block Scale should be 2D or 3D tensor.");
+  CHECK_CONTIGUOUS(blockScale);
+  CHECK_INPUT_TYPE(blockScale, dl_uint8);
+  auto blockScaleShape = blockScale.shape();
+  TVM_FFI_ICHECK(blockScaleShape.size() == 2 || blockScaleShape.size() == 3)
+      << "Block Scale should be 2D or 3D tensor.";
   auto num_experts = blockScaleShape.size() == 3 ? blockScaleShape[0] : 1;
   auto rows = blockScaleShape.size() == 3 ? blockScaleShape[1] : blockScaleShape[0];
   auto cols = blockScaleShape.size() == 3 ? blockScaleShape[2] : blockScaleShape[1];
@@ -155,25 +159,23 @@ at::Tensor BlockScaleInterleave(at::Tensor const& blockScale) {
   auto expert_out_size = tensorrt_llm::computeSwizzledLayoutSFSize(rows, cols);
   auto rows_padded = PadUpFn(rows, 128);
   auto cols_padded = PadUpFn(cols, 4);
-  TORCH_CHECK(expert_out_size == rows_padded * cols_padded,
-              "expert_out_size should be equal to rows_padded * cols_padded.");
-  at::Tensor interleavedBlockScale =
-      at::empty({expert_out_size * num_experts},
-                at::dtype(SF_DTYPE).device(blockScale.device()).requires_grad(false));
+  TVM_FFI_ICHECK(expert_out_size == rows_padded * cols_padded)
+      << "expert_out_size should be equal to rows_padded * cols_padded.";
 
   if (is_cuda) {
     const thread_local int smCount = tensorrt_llm::common::getMultiProcessorCount();
-    auto stream = at::cuda::getCurrentCUDAStream(blockScale.get_device());
+    const cudaStream_t stream = get_stream(blockScale->device);
+
     tensorrt_llm::kernels::invokeBlockScaleInterleave(
-        num_experts, rows, rows_padded, cols, cols_padded, blockScale.data_ptr<uint8_t>(),
-        static_cast<uint8_t*>(interleavedBlockScale.data_ptr()), smCount, stream);
+        num_experts, rows, rows_padded, cols, cols_padded, static_cast<uint8_t*>(blockScale->data),
+        static_cast<uint8_t*>(interleavedBlockScale->data), smCount, stream);
   } else {
     for (int eIdx = 0; eIdx < static_cast<int>(num_experts); eIdx++) {
       uint8_t* interleavedBlockScalePtr =
-          static_cast<uint8_t*>(interleavedBlockScale.data_ptr()) + eIdx * expert_out_size;
+          static_cast<uint8_t*>(interleavedBlockScale->data) + eIdx * expert_out_size;
       for (int rIdx = 0; rIdx < static_cast<int>(rows_padded); ++rIdx) {
         auto globalRowIdx = eIdx * rows + rIdx;
-        uint8_t* blockScalePtr = blockScale.data_ptr<uint8_t>() + globalRowIdx * cols;
+        uint8_t* blockScalePtr = static_cast<uint8_t*>(blockScale->data) + globalRowIdx * cols;
         for (int cIdx = 0; cIdx < static_cast<int>(cols_padded); ++cIdx) {
           uint8_t sf_ori = 0;
           if (rIdx < static_cast<int>(rows) && cIdx < static_cast<int>(cols)) {
@@ -186,146 +188,93 @@ at::Tensor BlockScaleInterleave(at::Tensor const& blockScale) {
       }
     }
   }
-
-  return interleavedBlockScale;
 }
 
 // Reverse interleave the weights block scaling factor.
 // blockScale: [num_experts, rows, cols] or [rows, cols]
 // Note: rows and cols are the dimensions of the original unswizzled SFMatrix, so reshape input
 // before passing into this function! Return: The same shape as blockScale
-at::Tensor BlockScaleInterleaveReverse(at::Tensor const& blockScale) {
-  bool is_cuda = blockScale.device().is_cuda();
+void BlockScaleInterleaveReverse(TensorView const& blockScale, TensorView reversedBlockScale) {
+  bool is_cuda = (blockScale->device.device_type == kDLCUDA);
   if (is_cuda) {
-    CHECK_INPUT_TYPE(blockScale, SF_DTYPE);
+    CHECK_CUDA(blockScale);
   } else {
-    CHECK_CPU_INPUT(blockScale, SF_DTYPE);
+    CHECK_CPU(blockScale);
   }
-  auto blockScaleShape = blockScale.sizes();
-  TORCH_CHECK(blockScaleShape.size() == 2 || blockScaleShape.size() == 3,
-              "Block Scale should be 2D or 3D tensor.");
+  CHECK_CONTIGUOUS(blockScale);
+  CHECK_INPUT_TYPE(blockScale, dl_uint8);
+  auto blockScaleShape = blockScale.shape();
+  TVM_FFI_ICHECK(blockScaleShape.size() == 2 || blockScaleShape.size() == 3)
+      << "Block Scale should be 2D or 3D tensor.";
   auto num_experts = blockScaleShape.size() == 3 ? blockScaleShape[0] : 1;
   auto rows = blockScaleShape.size() == 3 ? blockScaleShape[1] : blockScaleShape[0];
   auto cols = blockScaleShape.size() == 3 ? blockScaleShape[2] : blockScaleShape[1];
-  TORCH_CHECK(rows % 128 == 0, "rows of Interleaved block scales should be multiple of 128.");
-  TORCH_CHECK(cols % 4 == 0, "cols of Interleaved block scales should be multiple of 4.");
+  TVM_FFI_ICHECK(rows % 128 == 0) << "rows of Interleaved block scales should be multiple of 128.";
+  TVM_FFI_ICHECK(cols % 4 == 0) << "cols of Interleaved block scales should be multiple of 4.";
   auto expert_out_size = rows * cols;
-  at::Tensor reversedBlockScale = at::empty(
-      blockScaleShape, at::dtype(SF_DTYPE).device(blockScale.device()).requires_grad(false));
 
   if (is_cuda) {
     const thread_local int smCount = tensorrt_llm::common::getMultiProcessorCount();
-    auto stream = at::cuda::getCurrentCUDAStream(blockScale.get_device());
+    const cudaStream_t stream = get_stream(blockScale->device);
     tensorrt_llm::kernels::invokeBlockScaleInterleaveReverse(
-        num_experts, rows, cols, blockScale.data_ptr<uint8_t>(),
-        static_cast<uint8_t*>(reversedBlockScale.data_ptr()), smCount, stream);
+        num_experts, rows, cols, static_cast<uint8_t*>(blockScale->data),
+        static_cast<uint8_t*>(reversedBlockScale->data), smCount, stream);
   } else {
     // index in the swizzled SFMatrix -> (eIdx, rIdx, cIdx) in the unswizzled SFMatrix
-    std::map<int, std::array<int, 3>> identity;
+    Map<int, Array<int>> identity;
     for (int eIdx = 0; eIdx < num_experts; eIdx++) {
       for (int rIdx = 0; rIdx < rows; ++rIdx) {
         for (int cIdx = 0; cIdx < cols; ++cIdx) {
           int sf_index = computeSFIndex(rIdx, cIdx, rows, cols,
                                         tensorrt_llm::QuantizationSFLayout::SWIZZLED_128x4);
-          identity[eIdx * expert_out_size + sf_index] = std::array<int, 3>{eIdx, rIdx, cIdx};
+          identity.Set(eIdx * expert_out_size + sf_index, Array({eIdx, rIdx, cIdx}));
         }
       }
     }
-    uint8_t* blockScalePtr = static_cast<uint8_t*>(blockScale.data_ptr());
+    uint8_t* blockScalePtr = static_cast<uint8_t*>(blockScale->data);
     for (int i = 0; i < expert_out_size * num_experts; i++) {
       auto loc_2d = identity[i];
       if (loc_2d[1] < rows && loc_2d[2] < cols) {
-        uint8_t* reversedBlockScalePtr = reversedBlockScale.data_ptr<uint8_t>() +
+        uint8_t* reversedBlockScalePtr = static_cast<uint8_t*>(reversedBlockScale->data) +
                                          (loc_2d[0] * rows + loc_2d[1]) * cols + loc_2d[2];
         *reversedBlockScalePtr = blockScalePtr[i];
       }
     }
   }
-
-  return reversedBlockScale;
-}
-
-at::Tensor E2M1AndUFP8SFScaleToFloat(at::Tensor valueE2M1, at::Tensor scaleFP8SF, int64_t sfVecSize,
-                                     int64_t sfType) {
-  CHECK_CPU_INPUT(valueE2M1, FLOAT4_E2M1X2);
-  CHECK_CPU_INPUT(scaleFP8SF, SF_DTYPE);
-  auto packedShape = valueE2M1.sizes();
-  auto scaleShape = scaleFP8SF.sizes();
-  TORCH_CHECK(packedShape.size() == 2, "valueE2M1 should be 2D tensor.");
-  TORCH_CHECK(scaleShape.size() == 1, "scaleFP8SF should be 1D tensor.");
-  at::Tensor floatTensor = at::zeros({packedShape[0], packedShape[1] * 2},
-                                     at::dtype(at::ScalarType::Float).requires_grad(false));
-
-  int hiddenDim = packedShape[1] * 2;
-  int packedFp4HiddenDim = hiddenDim / 2;
-  int groupsPerHiddenDim = hiddenDim / sfVecSize;
-
-  for (size_t vIdx = 0; vIdx < static_cast<size_t>(packedShape[0]); ++vIdx) {
-    for (int group = 0; group < groupsPerHiddenDim; ++group) {
-      float* floatPtr = floatTensor.data_ptr<float>() + vIdx * hiddenDim + group * sfVecSize;
-      uint8_t* packedFp4Ptr =
-          valueE2M1.data_ptr<uint8_t>() + vIdx * packedFp4HiddenDim + group * sfVecSize / 2;
-      uint8_t* scaleFP8SFPtr = scaleFP8SF.data_ptr<uint8_t>();
-      uint8_t fp8Scale =
-          scaleFP8SFPtr[computeSFIndex(vIdx, group, packedShape[0], groupsPerHiddenDim,
-                                       tensorrt_llm::QuantizationSFLayout::SWIZZLED_128x4)];
-      int scale = fp8Scale;
-      if (sfType == 0) {
-        scale -= 127;
-      } else {
-        scale >>= 3;
-        scale -= 7;
-      }
-      float scaleFloat = makeExpFloat(scale);
-      for (int i = 0; i < sfVecSize; ++i) {
-        uint8_t packedFp4 = packedFp4Ptr[i / 2];
-        if (i % 2 == 1) {
-          packedFp4 >>= 4;
-        }
-        packedFp4 &= 0xf;
-        float value = e2M1ToFloat(packedFp4) * scaleFloat;
-        floatPtr[i] = value;
-      }
-    }
-  }
-  return floatTensor;
 }
 
 // Used by the (fp16 -> int4) quant layer + int4 gemm network.
-at::Tensor E2M1AndUFP8SFScaleToFloatV2(at::Tensor valueE2M1, at::Tensor scaleFP8SF,
-                                       std::optional<at::Tensor> globalScale, int64_t sfVecSize,
-                                       int64_t sfType, bool isSfSwizzledLayout = true) {
-  CHECK_CPU_INPUT(valueE2M1, FLOAT4_E2M1X2);
-  CHECK_CPU_INPUT(scaleFP8SF, SF_DTYPE);
-  auto packedShape = valueE2M1.sizes();
-  auto scaleShape = scaleFP8SF.sizes();
-  TORCH_CHECK(packedShape.size() == 2, "valueE2M1 should be 2D tensor.");
-  TORCH_CHECK(scaleShape.size() == 1, "scaleFP8SF should be 1D tensor.");
-  at::Tensor floatTensor = at::zeros({packedShape[0], packedShape[1] * 2},
-                                     at::dtype(at::ScalarType::Float).requires_grad(false));
+void E2M1AndUFP8SFScaleToFloatV2(TensorView valueE2M1, TensorView scaleFP8SF,
+                                 Optional<TensorView> globalScale, TensorView floatTensorView,
+                                 int64_t sfVecSize, int64_t sfType,
+                                 bool isSfSwizzledLayout = true) {
+  CHECK_CPU_INPUT(valueE2M1, dl_uint8);
+  CHECK_CPU_INPUT(scaleFP8SF, dl_uint8);
+  auto packedShape = valueE2M1.shape();
+  auto scaleShape = scaleFP8SF.shape();
+  TVM_FFI_ICHECK_EQ(packedShape.size(), 2) << "valueE2M1 should be 2D tensor.";
+  TVM_FFI_ICHECK_EQ(scaleShape.size(), 1) << "scaleFP8SF should be 1D tensor.";
 
-  // CHECK_CPU_INPUT(globalScale, at::ScalarType::Float);
   float globalScaleVal{1.0f};
   if (sfType == 1) {
-    TORCH_CHECK(globalScale.has_value(), "globalScale is required when sfType is 1.");
-    // CHECK_CPU_INPUT(globalScale.value(), at::kFloat32);
-    globalScaleVal = globalScale->data_ptr<float>()[0];
+    TVM_FFI_ICHECK(globalScale.has_value()) << "globalScale is required when sfType is 1.";
+    globalScaleVal = static_cast<float*>(globalScale.value()->data)[0];
   }
 
   int hiddenDim = packedShape[1] * 2;
   int packedFp4HiddenDim = hiddenDim / 2;
   int groupsPerHiddenDim = hiddenDim / sfVecSize;
-
   tensorrt_llm::QuantizationSFLayout layout =
       isSfSwizzledLayout ? tensorrt_llm::QuantizationSFLayout::SWIZZLED_128x4
                          : tensorrt_llm::QuantizationSFLayout::LINEAR;
 
   for (size_t vIdx = 0; vIdx < static_cast<size_t>(packedShape[0]); ++vIdx) {
     for (int group = 0; group < groupsPerHiddenDim; ++group) {
-      float* floatPtr = floatTensor.data_ptr<float>() + vIdx * hiddenDim + group * sfVecSize;
-      uint8_t* packedFp4Ptr =
-          valueE2M1.data_ptr<uint8_t>() + vIdx * packedFp4HiddenDim + group * sfVecSize / 2;
-      uint8_t* scaleFP8SFPtr = scaleFP8SF.data_ptr<uint8_t>();
+      float* floatPtr =
+          static_cast<float*>(floatTensorView->data) + vIdx * hiddenDim + group * sfVecSize;
+      uint8_t* packedFp4Ptr = static_cast<uint8_t*>(valueE2M1->data) + vIdx * packedFp4HiddenDim +
+                              group * sfVecSize / 2;
+      uint8_t* scaleFP8SFPtr = static_cast<uint8_t*>(scaleFP8SF->data);
       uint8_t fp8Scale =
           scaleFP8SFPtr[computeSFIndex(vIdx, group, packedShape[0], groupsPerHiddenDim, layout)];
       float scaleFloat;
@@ -347,40 +296,37 @@ at::Tensor E2M1AndUFP8SFScaleToFloatV2(at::Tensor valueE2M1, at::Tensor scaleFP8
       }
     }
   }
-  return floatTensor;
 }
 
-at::Tensor mxfp4_dequantize_host(at::Tensor weight, at::Tensor scale, int64_t group_size) {
+void mxfp4_dequantize_host(TensorView weight, TensorView scale, TensorView dequant_weight,
+                           int64_t group_size) {
   // weight (n, k / 2)
   // scale (n, k / group_size)
 
-  CHECK_CPU_INPUT(weight, FLOAT4_E2M1X2);
-  CHECK_CPU_INPUT(scale, SF_DTYPE);
-  TORCH_CHECK(weight.is_contiguous(), "weight must be contiguous");
-  TORCH_CHECK(scale.is_contiguous(), "scale must be contiguous");
-  TORCH_CHECK(weight.numel() != 0, "weight should not be empty tensor");
-  TORCH_CHECK(weight.dtype() == at::ScalarType::Byte, "Weight must be a packed int8 tensor");
-  TORCH_CHECK(scale.dtype() == at::ScalarType::Byte, "Scale must be a int8 tensor");
+  CHECK_CPU_INPUT(weight, dl_uint8);
+  CHECK_CPU_INPUT(scale, dl_uint8);
+  CHECK_CONTIGUOUS(weight);
+  CHECK_CONTIGUOUS(scale);
+  TVM_FFI_ICHECK_NE(weight.shape().Product(), 0) << "weight should not be empty tensor";
+  CHECK_INPUT_TYPE(weight, dl_uint8);
+  CHECK_INPUT_TYPE(scale, dl_uint8);
 
-  TORCH_CHECK(weight.size(0) == scale.size(0),
-              "weight and scale must have the same number of rows");
-  TORCH_CHECK(weight.size(1) * 2 == scale.size(1) * group_size,
-              "weight and scale must have the same number of columns");
+  int const n = weight->shape[0];
+  int const k = weight->shape[1] * 2;
 
-  uint8_t* weight_packed_ptr = weight.data_ptr<uint8_t>();
-  __nv_fp8_e8m0* scale_ptr = reinterpret_cast<__nv_fp8_e8m0*>(scale.data_ptr<uint8_t>());
-
-  int const n = weight.size(0);
-  int const k = weight.size(1) * 2;
-
-  at::Tensor dequant_weight =
-      at::empty({n, k}, at::dtype(at::ScalarType::Float).device(at::kCPU).requires_grad(false));
-  float* dequant_weight_ptr = dequant_weight.data_ptr<float>();
+  TVM_FFI_ICHECK_EQ(n, scale->shape[0]) << "weight and scale must have the same number of rows";
+  TVM_FFI_ICHECK_EQ(k, scale->shape[1] * group_size)
+      << "weight and scale must have the same number of columns";
 
+#if CUDA_VERSION >= 12080
+  uint8_t* weight_packed_ptr = static_cast<uint8_t*>(weight->data);
+  __nv_fp8_e8m0* scale_ptr = reinterpret_cast<__nv_fp8_e8m0*>(static_cast<uint8_t*>(scale->data));
+  float* dequant_weight_ptr = static_cast<float*>(dequant_weight->data);
   float fp4_lut[] = {0.0, 0.5,  1.0,  1.5,  2.0,  3.0,  4.0,  6.0,
                      0.0, -0.5, -1.0, -1.5, -2.0, -3.0, -4.0, -6.0};
 
-  for (int packed_idx = 0; packed_idx < weight.numel(); ++packed_idx) {
+  const auto num_packed_elements = weight.shape().Product();
+  for (int packed_idx = 0; packed_idx < num_packed_elements; ++packed_idx) {
     int8_t weight_packed_data = weight_packed_ptr[packed_idx];
 
     uint8_t weight_low_ = weight_packed_data & 0xF;
@@ -392,20 +338,18 @@ at::Tensor mxfp4_dequantize_host(at::Tensor weight, at::Tensor scale, int64_t gr
     int scale_n_idx = packed_idx / (k / 2);
     int scale_k_idx = ((packed_idx * 2) % k) / group_size;
 
-    float scale_ = static_cast<float>(scale_ptr[scale_n_idx * scale.size(1) + scale_k_idx]);
+    float scale_ = static_cast<float>(scale_ptr[scale_n_idx * scale->shape[1] + scale_k_idx]);
 
     dequant_weight_ptr[2 * packed_idx] = weight_low * scale_;
     dequant_weight_ptr[2 * packed_idx + 1] = weight_high * scale_;
   }
-
-  return dequant_weight;
+#else
+  // NOTE(Zihao): __nv_fp8_e8m0 is not supported for CUDA < 12.8
+  TLLM_THROW("mxfp4_dequantize_host is not supported for CUDA < 12.8");
+#endif
 }
 
-}  // namespace torch_ext
-
-TORCH_LIBRARY_FRAGMENT(TORCH_EXTENSION_NAME, m) {
-  m.def("block_scale_interleave_sm100", &torch_ext::BlockScaleInterleave);
-  m.def("block_scale_interleave_reverse_sm100", &torch_ext::BlockScaleInterleaveReverse);
-  m.def("e2m1_and_ufp8sf_scale_to_float_sm100", &torch_ext::E2M1AndUFP8SFScaleToFloatV2);
-  m.def("mxfp4_dequantize_host", &torch_ext::mxfp4_dequantize_host);
-}
+TVM_FFI_DLL_EXPORT_TYPED_FUNC(block_scale_interleave_sm100, BlockScaleInterleave);
+TVM_FFI_DLL_EXPORT_TYPED_FUNC(block_scale_interleave_reverse_sm100, BlockScaleInterleaveReverse);
+TVM_FFI_DLL_EXPORT_TYPED_FUNC(e2m1_and_ufp8sf_scale_to_float_sm100, E2M1AndUFP8SFScaleToFloatV2);
+TVM_FFI_DLL_EXPORT_TYPED_FUNC(mxfp4_dequantize_host, mxfp4_dequantize_host);
