@@ -27,7 +27,11 @@ from .utils import (
     TensorLayout,
     _check_kv_layout,
     _unpack_paged_kv_cache,
+    determine_attention_backend,
 )
+from .prefill import BatchPrefillWithPagedKVCacheWrapper
+from .jit.attention.variants import attention_sink_decl
+from .jit.utils import filename_safe_dtype_map
 
 
 @functools.cache
@@ -45,7 +49,7 @@ class BatchAttention:
         self._kv_layout = kv_layout
 
         self.float_workspace_buffer = torch.empty(
-            256 * 1024 * 1024,
+            384 * 1024 * 1024,
             dtype=torch.uint8,
             device=torch.device(device),
         )
@@ -109,13 +113,11 @@ class BatchAttention:
         self._num_qo_heads = num_qo_heads
         self._num_kv_heads = num_kv_heads
         self._page_size = page_size
-        self._sm_scale = sm_scale
         self._use_profiler = use_profiler
 
         # No addtional buf allocated for CUDA graph tensor
         # Allocate outside FlashInfer
         self._kv_indices = kv_indices
-
         self._plan_info = self.module.plan(
             self.float_workspace_buffer,
             self.int_workspace_buffer,
@@ -136,6 +138,8 @@ class BatchAttention:
         kv_cache: Union[torch.Tensor, Tuple[torch.Tensor, torch.Tensor]],
         out: Optional[torch.Tensor] = None,
         lse: Optional[torch.Tensor] = None,
+        k_scale: Optional[torch.Tensor] = None,
+        v_scale: Optional[torch.Tensor] = None,
         logits_soft_cap: float = 0.0,
         profiler_buffer: Optional[torch.Tensor] = None,
     ) -> Tuple[torch.Tensor, torch.Tensor]:
@@ -154,12 +158,17 @@ class BatchAttention:
             out = torch.empty_like(q)
         if lse is None:
             # lse shape: [batch_size, num_qo_heads]
-            lse = torch.empty(q.shape[0], q.shape[1], device=q.device)
-
+            lse = torch.empty(
+                q.shape[0], q.shape[1], device=q.device, dtype=torch.float32
+            )
         head_dim_qk = q.shape[2]
-        if self._sm_scale is None:
-            self._sm_scale = 1.0 / math.sqrt(head_dim_qk)
-
+        sm_scale = self._sm_scale
+        if sm_scale is None:
+            sm_scale = 1.0 / math.sqrt(head_dim_qk)
+        if k_scale is not None:
+            sm_scale *= k_scale
+        if v_scale is None:
+            v_scale = 1.0
         # profiler_buffer is optional
         profiler_args = (profiler_buffer,) if self._use_profiler else ()
 
@@ -178,7 +187,8 @@ class BatchAttention:
             self._num_qo_heads,
             self._num_kv_heads,
             self._page_size,
-            self._sm_scale,
+            v_scale,
+            sm_scale,
             logits_soft_cap,
             # ADDITIONAL_FUNC_PARAMS
             # PROFILER_FUNC_PARAMS
@@ -186,3 +196,80 @@ class BatchAttention:
         )
 
         return out, lse
+
+
+class BatchAttentionWithAttentionSinkWrapper(BatchPrefillWithPagedKVCacheWrapper):
+    r"""
+    Wrapper for prefill and decode attention with paged KV-cache that adds support for
+    attention sinks. This class extends `BatchPrefillWithPagedKVCacheWrapper`, providing
+    a convenient interface for using attention sinks during prefill or decode attention.
+    """
+
+    def __init__(
+        self,
+        float_workspace_buffer: torch.Tensor,
+        kv_layout: str = "NHD",
+        use_cuda_graph: bool = False,
+        qo_indptr_buf: Optional[torch.Tensor] = None,
+        paged_kv_indptr_buf: Optional[torch.Tensor] = None,
+        paged_kv_indices_buf: Optional[torch.Tensor] = None,
+        paged_kv_last_page_len_buf: Optional[torch.Tensor] = None,
+        custom_mask_buf: Optional[torch.Tensor] = None,
+        mask_indptr_buf: Optional[torch.Tensor] = None,
+        backend: str = "auto",
+        pos_encoding_mode: str = "NONE",
+        use_fp16_qk_reduction: bool = False,
+        q_data_type: torch.dtype = torch.bfloat16,
+        kv_data_type: torch.dtype = torch.bfloat16,
+        head_dim_qk: int = 128,
+        head_dim_vo: int = 128,
+        window_left: int = -1,
+    ) -> None:
+        # trtllm is separate code path
+        assert backend in ["fa2", "fa3", "auto"]
+        if backend == "auto":
+            # dispatch backend before init jit module
+            backend = determine_attention_backend(
+                float_workspace_buffer.device,
+                PosEncodingMode[pos_encoding_mode].value,
+                use_fp16_qk_reduction,  # use_fp16_qk_reduction
+                custom_mask_buf is not None,  # use_custom_mask
+                q_data_type,
+                kv_data_type,
+            )
+
+        jit_args = [
+            f"batch_prefill_attention_sink_{filename_safe_dtype_map[q_data_type]}_swa_{window_left >= 0}_{backend}",  # uri
+            q_data_type,  # dtype_q
+            kv_data_type,  # dtype_kv
+            q_data_type,  # dtype_o
+            torch.int32,  # idtype
+            head_dim_qk,  # hidden_dim_qk
+            head_dim_vo,  # hidden_dim_vo
+            ["sink"],  # additional_tensor_names
+            ["float"],  # additional_tensor_dtypes
+            ["sm_scale"],  # additional_scalar_names
+            ["double"],  # additional_scalar_dtypes
+            "AttentionSink",
+            attention_sink_decl[backend],
+        ]
+        jit_kwargs = {
+            "use_sliding_window": window_left >= 0,
+            "use_fp16_qk_reduction": use_fp16_qk_reduction,
+            "pos_encoding_mode": PosEncodingMode[pos_encoding_mode].value,
+        }
+
+        super().__init__(
+            float_workspace_buffer=float_workspace_buffer,
+            kv_layout=kv_layout,
+            use_cuda_graph=use_cuda_graph,
+            qo_indptr_buf=qo_indptr_buf,
+            paged_kv_indptr_buf=paged_kv_indptr_buf,
+            paged_kv_indices_buf=paged_kv_indices_buf,
+            paged_kv_last_page_len_buf=paged_kv_last_page_len_buf,
+            custom_mask_buf=custom_mask_buf,
+            mask_indptr_buf=mask_indptr_buf,
+            backend=backend,
+            jit_args=jit_args,
+            jit_kwargs=jit_kwargs,
+        )
