@@ -10,47 +10,27 @@ from packaging.version import Version
 from pathlib import Path
 from typing import List, Optional
 
+import tvm_ffi
 import torch
-from torch.utils.cpp_extension import (
-    _TORCH_PATH,
-    CUDA_HOME,
-    _get_num_workers,
-    _get_pybind11_abi_build_flags,
-)
 
 from . import env as jit_env
 from ..compilation_context import CompilationContext
 
 
-@functools.cache
-def get_cuda_path() -> str:
-    if CUDA_HOME is None:
-        # get output of "which nvcc"
-        result = subprocess.run(["which", "nvcc"], capture_output=True)
-        if result.returncode != 0:
-            raise RuntimeError("Could not find nvcc")
-        return result.stdout.decode("utf-8").strip()
-    else:
-        return CUDA_HOME
+def parse_env_flags(env_var_name) -> List[str]:
+    env_flags = os.environ.get(env_var_name)
+    if env_flags:
+        try:
+            import shlex
 
-
-@functools.cache
-def get_cuda_version() -> Version:
-    if CUDA_HOME is None:
-        nvcc = "nvcc"
-    else:
-        nvcc = os.path.join(CUDA_HOME, "bin/nvcc")
-    txt = subprocess.check_output([nvcc, "--version"], text=True)
-    matches = re.findall(r"release (\d+\.\d+),", txt)
-    if not matches:
-        raise RuntimeError(
-            f"Could not parse CUDA version from nvcc --version output: {txt}"
-        )
-    return Version(matches[0])
-
-
-def is_cuda_version_at_least(version_str: str) -> bool:
-    return get_cuda_version() >= Version(version_str)
+            return shlex.split(env_flags)
+        except ValueError as e:
+            print(
+                f"Warning: Could not parse {env_var_name} with shlex: {e}. Falling back to simple split.",
+                file=sys.stderr,
+            )
+            return env_flags.split()
+    return []
 
 
 def _get_glibcxx_abi_build_flags() -> List[str]:
@@ -60,6 +40,48 @@ def _get_glibcxx_abi_build_flags() -> List[str]:
     return glibcxx_abi_cflags
 
 
+@functools.cache
+def get_cuda_path() -> str:
+    cuda_home = os.environ.get("CUDA_HOME") or os.environ.get("CUDA_PATH")
+    if cuda_home is not None:
+        return cuda_home
+    # get output of "which nvcc"
+    nvcc_path = subprocess.run(["which", "nvcc"], capture_output=True)
+    if nvcc_path.returncode != 0:
+        raise RuntimeError("Could not find nvcc")
+    cuda_home = os.path.dirname(
+        os.path.dirname(nvcc_path.stdout.decode("utf-8").strip())
+    )
+    return cuda_home
+
+
+@functools.cache
+def get_cuda_version() -> Version:
+    # Try to query nvcc for CUDA version; if nvcc is unavailable, fall back to torch.version.cuda
+    try:
+        cuda_home = get_cuda_path()
+        nvcc = os.path.join(cuda_home, "bin/nvcc")
+        txt = subprocess.check_output([nvcc, "--version"], text=True)
+        matches = re.findall(r"release (\d+\.\d+),", txt)
+        if not matches:
+            raise RuntimeError(
+                f"Could not parse CUDA version from nvcc --version output: {txt}"
+            )
+        return Version(matches[0])
+    except (RuntimeError, FileNotFoundError, subprocess.CalledProcessError) as e:
+        # NOTE(Zihao): when nvcc is unavailable, fall back to torch.version.cuda
+        if torch.version.cuda is None:
+            raise RuntimeError(
+                "nvcc not found and PyTorch is not built with CUDA support. "
+                "Could not determine CUDA version."
+            ) from e
+        return Version(torch.version.cuda)
+
+
+def is_cuda_version_at_least(version_str: str) -> bool:
+    return get_cuda_version() >= Version(version_str)
+
+
 def join_multiline(vs: List[str]) -> str:
     return " $\n    ".join(vs)
 
@@ -75,22 +97,24 @@ def generate_ninja_build_for_op(
 ) -> str:
     system_includes = [
         sysconfig.get_path("include"),
-        "$torch_home/include",
-        "$torch_home/include/torch/csrc/api/include",
         "$cuda_home/include",
         "$cuda_home/include/cccl",
+        tvm_ffi.libinfo.find_include_path(),
+        tvm_ffi.libinfo.find_dlpack_include_path(),
         jit_env.FLASHINFER_INCLUDE_DIR.resolve(),
         jit_env.FLASHINFER_CSRC_DIR.resolve(),
     ]
     system_includes += [p.resolve() for p in jit_env.CUTLASS_INCLUDE_DIRS]
     system_includes.append(jit_env.SPDLOG_INCLUDE_DIR.resolve())
 
-    common_cflags = [
-        "-DTORCH_EXTENSION_NAME=$name",
-        "-DTORCH_API_INCLUDE_EXTENSION_H",
-        "-DPy_LIMITED_API=0x03090000",
-    ]
-    common_cflags += _get_pybind11_abi_build_flags()
+    cuda_home = get_cuda_path()
+    if cuda_home == "/usr":
+        # NOTE: this will resolve to /usr/include, which will mess up includes. See #1793
+        system_includes.remove("$cuda_home/include")
+
+    common_cflags = []
+    if not sysconfig.get_config_var("Py_GIL_DISABLED"):
+        common_cflags.append("-DPy_LIMITED_API=0x03090000")
     common_cflags += _get_glibcxx_abi_build_flags()
     if extra_include_dirs is not None:
         for extra_dir in extra_include_dirs:
@@ -144,41 +168,34 @@ def generate_ninja_build_for_op(
 
     ldflags = [
         "-shared",
-        "-L$torch_home/lib",
         "-L$cuda_home/lib64",
-        "-lc10",
-        "-lc10_cuda",
-        "-ltorch_cpu",
-        "-ltorch_cuda",
-        "-ltorch",
+        "-L$cuda_home/lib64/stubs",
         "-lcudart",
+        "-lcuda",
     ]
 
-    env_extra_ldflags = os.environ.get("FLASHINFER_EXTRA_LDFLAGS")
-    if env_extra_ldflags:
-        try:
-            import shlex
-
-            ldflags += shlex.split(env_extra_ldflags)
-        except ValueError as e:
-            print(
-                f"Warning: Could not parse FLASHINFER_EXTRA_LDFLAGS with shlex: {e}. Falling back to simple split.",
-                file=sys.stderr,
-            )
-            ldflags += env_extra_ldflags.split()
+    env_extra_ldflags = parse_env_flags("FLASHINFER_EXTRA_LDFLAGS")
+    if env_extra_ldflags is not None:
+        ldflags += env_extra_ldflags
 
     if extra_ldflags is not None:
         ldflags += extra_ldflags
 
+    extra_cflags = parse_env_flags("FLASHINFER_EXTRA_CFLAGS")
+    if extra_cflags is not None:
+        cflags += extra_cflags
+
+    extra_cuda_cflags = parse_env_flags("FLASHINFER_EXTRA_CUDAFLAGS")
+    if extra_cuda_cflags is not None:
+        cuda_cflags += extra_cuda_cflags
+
     cxx = os.environ.get("CXX", "c++")
-    cuda_home = CUDA_HOME or "/usr/local/cuda"
-    nvcc = os.environ.get("PYTORCH_NVCC", "$cuda_home/bin/nvcc")
+    nvcc = os.environ.get("FLASHINFER_NVCC", "$cuda_home/bin/nvcc")
 
     lines = [
         "ninja_required_version = 1.3",
         f"name = {name}",
         f"cuda_home = {cuda_home}",
-        f"torch_home = {_TORCH_PATH}",
         f"cxx = {cxx}",
         f"nvcc = {nvcc}",
         "",
@@ -238,6 +255,13 @@ def generate_ninja_build_for_op(
     return "\n".join(lines)
 
 
+def _get_num_workers() -> Optional[int]:
+    max_jobs = os.environ.get("MAX_JOBS")
+    if max_jobs is not None and max_jobs.isdigit():
+        return int(max_jobs)
+    return None
+
+
 def run_ninja(workdir: Path, ninja_file: Path, verbose: bool) -> None:
     workdir.mkdir(parents=True, exist_ok=True)
     command = [
@@ -248,7 +272,7 @@ def run_ninja(workdir: Path, ninja_file: Path, verbose: bool) -> None:
         "-f",
         str(ninja_file.resolve()),
     ]
-    num_workers = _get_num_workers(verbose)
+    num_workers = _get_num_workers()
     if num_workers is not None:
         command += ["-j", str(num_workers)]
 
