@@ -24,10 +24,11 @@
 #include "flashinfer/trtllm/batched_gemm/trtllmGen_bmm_export/trtllm/gen/DtypeDecl.h"
 #include "flashinfer/trtllm/batched_gemm/trtllmGen_bmm_export/trtllm/gen/SfLayoutDecl.h"
 // #include <cuda_runtime_api.h>
-#include <c10/util/Exception.h>
 #include <cutlass/cutlass.h>
 #include <cutlass/numeric_size.h>
 #include <cutlass/numeric_types.h>
+
+#include "../../exception.h"
 // #include <tensorrt_llm/common/assert.h>
 #include "flashinfer/trtllm/common/cudaUtils.h"
 
@@ -40,7 +41,7 @@ namespace moe::dev {
       std::cout << "CUDA error in " << __FILE__ << ":" << __LINE__ << " executing '" << #cmd \
                 << "': " << cudaGetErrorString(e);                                           \
     }                                                                                        \
-    TORCH_CHECK(e == cudaSuccess, "Got CUDA error. See above for details.");                 \
+    FLASHINFER_CHECK(e == cudaSuccess, "Got CUDA error. See above for details.");            \
   } while (0)
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
@@ -85,7 +86,7 @@ namespace moe::dev {
   } else if (data.mDtypeElt == tg::Dtype::Bfloat16) {                                              \
     LAUNCH_PDL(data, false, cutlass::bfloat16_t, kernel, numBlocks, numThreads, smemSize, stream); \
   } else {                                                                                         \
-    TORCH_WARN("Unsupported dtypeElt");                                                            \
+    FLASHINFER_WARN("Unsupported dtypeElt");                                                       \
   }
 
 #define LAUNCH_EXPW(data, kernel, numBlocks, numThreads, smemSize, stream)                         \
@@ -108,7 +109,7 @@ namespace moe::dev {
     LAUNCH_PDL(data, false, LAUNCH_ESC(cutlass::bfloat16_t, cutlass::bfloat16_t), kernel,          \
                numBlocks, numThreads, smemSize, stream);                                           \
   } else {                                                                                         \
-    TORCH_WARN("Unsupported pair");                                                                \
+    FLASHINFER_WARN("Unsupported pair");                                                           \
   }
 
 #define LAUNCH_ROUTING(data, coopLaunch, kernel, numBlocks, numThreads, smemSize, stream)     \
@@ -119,7 +120,7 @@ namespace moe::dev {
     LAUNCH_PDL(data, coopLaunch, LAUNCH_ESC(__nv_bfloat16, __nv_bfloat16), kernel, numBlocks, \
                numThreads, smemSize, stream);                                                 \
   } else {                                                                                    \
-    TORCH_WARN("Unsupported dtypeExpW");                                                      \
+    FLASHINFER_WARN("Unsupported dtypeExpW");                                                 \
   }
 
 #define LAUNCH_ROUTING_WITH_EXTRA_FLAG(data, coopLaunch, kernel, numBlocks, numThreads, smemSize, \
@@ -143,7 +144,7 @@ namespace moe::dev {
     LAUNCH_PDL(data, coopLaunch, LAUNCH_ESC(__nv_bfloat16, __nv_bfloat16, false), kernel,         \
                numBlocks, numThreads, smemSize, stream);                                          \
   } else {                                                                                        \
-    TORCH_WARN("Unsupported dtypeExpW");                                                          \
+    FLASHINFER_WARN("Unsupported dtypeExpW");                                                     \
   }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
